package cn.com.ty.zuul.filter;

import cn.com.ty.zuul.common.Constants;
import cn.com.ty.zuul.security.utils.VerifyUtils;
import cn.com.ty.zuul.utils.ProjectUtils;
import cn.com.ty.zuul.vo.RestResponse;
import cn.hutool.json.JSONUtil;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author huangyuan
 * @date 2019-11-27
 * @description 用户登录拦截器
 */
@Component
@Slf4j
public class LoginFilter extends ZuulFilter {

    @Resource
    private RedisTemplate redisTemplate;

    @Override
    public String filterType() {
        return "pre";
    }

    @Override
    public int filterOrder() {
        return 1;
    }

    @Override
    public boolean shouldFilter() {
        return true;
    }

    @Override
    public Object run() {

        //获取当前session信息
        RequestContext ctx = RequestContext.getCurrentContext();
        HttpServletRequest request = ctx.getRequest();
        String requestPath = VerifyUtils.getRequestPath(request);
        log.error("请求路径" + requestPath);

        //处理websocket请求问题
        if (Constants.WEBSOCKET_URL.stream().anyMatch(requestPath::matches)) {
            //请求头中添加Upgrade
            ctx.addZuulRequestHeader("connection", "Upgrade");
            log.error("websocket连接");
            return null;
        }

        //公共访问路径过滤
        if (Constants.UPLOAD_IMAGES_URLS.stream().anyMatch(requestPath::matches)) {
            return null;
        }
        //判断访问是否合法
        //查看访问来源
        String mobileType = request.getHeader(Constants.AccessType.ACCESS_TYPE_FIELD);
        //如果请求头中没有mobileType就属于不合法访问
        if (StringUtils.isBlank(mobileType)) {
            returnMessage(ctx, "非法请求请重试", Constants.StatusCode.RESULT_ILLEGAL_REQUEST);
            return null;
        }
        //将请求端类型放入请求参数中
        ProjectUtils.setMobileTypeIntoRequest(ctx, mobileType);

        //公共接口不需要登录
        boolean isCommonIgnoreUrl = Constants.COMMON_IGNORE_URLS.stream().anyMatch(requestPath::matches);
        if (isCommonIgnoreUrl) {
            return null;
        }

        //获取用户授权码
        String userInfoToken = request.getHeader(Constants.AccessType.USER_INFO_TOKEN);
        //用户信息和不同的登录端绑定， 通过用户授权码和登录的端来获取用户信息
        Object mobileUserInfo = redisTemplate.opsForValue().get(userInfoToken);
        if (mobileUserInfo == null) {
            returnMessage(ctx, "用户尚未登录", Constants.StatusCode.RESULT_NO_LOGIN);
            return null;
        }
        //如果用户信息为空，提示未登录
        Object userInfo = redisTemplate.opsForValue().get(mobileUserInfo);
        if (userInfo == null) {
            returnMessage(ctx, "用户尚未登录", Constants.StatusCode.RESULT_NO_LOGIN);
            return null;
        }
        //1.获取redis中的用户信息判断用户是否登录
        Map<String, Object> currentUserInfoMap = JSONUtil.parseObj(userInfo);
        //判断用户map中是否有空值，有空值判断用户登录异常
        if (ProjectUtils.verifyMapHasNullValue(currentUserInfoMap)) {
            returnMessage(ctx, "用户登录异常", Constants.StatusCode.RESULT_ERROR);
            return null;
        }

        //2.用户登录成功 对用户做权限校验
//        if (!VerifyUtils.verifyUserHasPermission(requestPath, currentUserInfoMap, Integer.parseInt(mobileType))) {
//            returnMessage(ctx, "用户权限不足", Constants.StatusCode.RESULT_NO_ACCESS);
//            return null;
//        }
        //修改request中的参数
        ProjectUtils.setUserInfoParameterIntoRequest(ctx, currentUserInfoMap, userInfoToken,
                Integer.parseInt(mobileType));

        //在ctx头中放数据
        ctx.addZuulRequestHeader(Constants.CURRENT_USER_ID, currentUserInfoMap.get(Constants.CURRENT_USER_ID).toString());
        ctx.addZuulRequestHeader(Constants.CURRENT_COMPANY_ID, currentUserInfoMap.get(Constants.CURRENT_COMPANY_ID).toString());

        //重置用户登录信息在redis中的时间
        if (!(Constants.AccessType.APP_ACCESS == Integer.parseInt(mobileType))) {
            redisTemplate.opsForValue().set(userInfoToken, mobileUserInfo, 30L, TimeUnit.MINUTES);
            //获取用户信息，将redis中的用户信息刷新
            redisTemplate.opsForValue().set(mobileUserInfo, JSONUtil.toJsonPrettyStr(currentUserInfoMap),
                    30L, TimeUnit.MINUTES);
        }
        return null;
    }

    /**
     * @param ctx     请求信息
     * @param message 返回信息
     * @return void
     * @description 校验不通过，返回信息
     * @date 2020-01-16 15:33
     */
    private void returnMessage(RequestContext ctx, String message, String resultCode) {
        try {
            //用户未登录，提示用户未登录
            RestResponse restResponse = new RestResponse(resultCode, message);
            ProjectUtils.returnMsg(ctx, 401, restResponse, Constants.ContentType.JSON);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
