package cn.com.ty.zuul.utils;

import cn.com.ty.zuul.common.Constants;
import cn.com.ty.zuul.security.utils.VerifyUtils;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.netflix.zuul.context.RequestContext;
import com.netflix.zuul.http.ServletInputStreamWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.StreamUtils;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequestWrapper;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Collection;
import java.util.Map;

/**
 * @author huangyuan
 * @date 2019-11-28
 * @description 项目自用工具类
 */
public class ProjectUtils {

    /**
     * @param ctx         响应对象
     * @param statusCode  响应状态码
     * @param message     响应信息
     * @param contentType 响应信息类型
     * @description 返回消息
     * @date 2019/11/27 10:03 AM
     */
    public static void returnMsg(RequestContext ctx, Integer statusCode, Object message, String contentType) {
        ctx.setResponseStatusCode(statusCode);
        ctx.setResponseBody(JSONUtil.parseObj(message).toString());
        ctx.getResponse().setContentType(contentType);
        try (InputStream stream = ctx.getResponseDataStream()) {
            String body = StreamUtils.copyToString(stream, StandardCharsets.UTF_8);
            ctx.setResponseDataStream(new ByteArrayInputStream(body.getBytes()));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * @param ctx 请求
     * @return mobileType 请求端
     * @description 封装请求端到请求数据中
     * @date 2020/4/22 3:08 下午
     */
    public static void setMobileTypeIntoRequest(RequestContext ctx, String mobileType) {
        //获取ctx中原有的请求参数
        JSONObject jsonObject = getRequestParameter(ctx);
        jsonObject.put(Constants.FieldConstants.MOBILE_TYPE, mobileType);
        //将参数放入request请求中
        setParameterIntoRequest(ctx, jsonObject);
    }

    /**
     * @param ctx           请求体对象
     * @param userInfoMap   用户信息:当前用户id，当前公司id
     * @param userInfoToken 用户授权码
     * @param mobileType 端类型
     * @description 往请求体中添加参数
     */
    public static void setUserInfoParameterIntoRequest(RequestContext ctx, Map<String, Object> userInfoMap,
                                                       String userInfoToken, int mobileType) {
        //获取请求路径
        String requestPath = VerifyUtils.getRequestPath(ctx.getRequest());
        JSONObject jsonObject = getRequestParameter(ctx);
        //往参数中添加当前用户id
        //查看请求路劲是否是应该忽略设置当前用户id的
        if (Constants.USER_ID_IGNORE_URLS.stream().noneMatch(requestPath::matches)) {
            jsonObject.put(Constants.FieldConstants.USER_ID_FIELD, userInfoMap.get(Constants.CURRENT_USER_ID));
        }
        //往参数中添加当前公司id
        //物管端用户不进行公司和角色信息的设置
        if(Constants.AccessType.THE_PUBLIC_ACCESS != mobileType) {
            //查看请求路劲是否是应该忽略设置当前公司id的
            if (Constants.COMPANY_ID_IGNORE_URLS.stream().noneMatch(requestPath::matches)) {
                jsonObject.put(Constants.FieldConstants.COMPANY_ID, userInfoMap.get(Constants.CURRENT_COMPANY_ID));
                jsonObject.put(Constants.FieldConstants.MT_COMPANY_ID, userInfoMap.get(Constants.CURRENT_COMPANY_ID));
            }
            //在参数中设置角色编码-用户获取企业待办数据中
            jsonObject.put(Constants.FieldConstants.CURRENT_ROLE_CODE, userInfoMap.get(Constants.CURRENT_ROLE_CODE));
        }
        //在参数中设置token值
        jsonObject.put(Constants.CURRENT_USER_TOKEN, userInfoToken);
        //设置当前用户id
        jsonObject.put(Constants.CURRENT_USER_ID, userInfoMap.get(Constants.CURRENT_USER_ID));
        //设置当前公司id
        jsonObject.put(Constants.CURRENT_COMPANY_ID, userInfoMap.get(Constants.CURRENT_COMPANY_ID));
        //将参数放入request请求中
        setParameterIntoRequest(ctx, jsonObject);
    }

    /**
     * @param ctx 请求数据
     * @return 返回之前的参数数据
     * @description
     * @date 2020/4/22 2:52 下午
     */
    private static JSONObject getRequestParameter(RequestContext ctx) {
        String body = null;
        try (InputStream in = ctx.getRequest().getInputStream()) {
            //拷贝原来的参数
            body = StreamUtils.copyToString(in, StandardCharsets.UTF_8);
        } catch (IOException e) {
            e.printStackTrace();
        }
        if (StringUtils.isBlank(body)) {
            body = "{}";
        }
        return JSONUtil.parseObj(body);
    }

    /**
     * @param ctx        网关请求对象
     * @param jsonObject 要放入的参数
     * @description
     * @date 2020/4/22 9:36 上午
     */
    private static void setParameterIntoRequest(RequestContext ctx, JSONObject jsonObject) {
        String newBody = jsonObject.toString();
        final byte[] reqBodyBytes = newBody.getBytes();
        ctx.setRequest(new HttpServletRequestWrapper(ctx.getRequest()) {
            @Override
            public ServletInputStream getInputStream() {
                return new ServletInputStreamWrapper(reqBodyBytes);
            }

            @Override
            public int getContentLength() {
                return reqBodyBytes.length;
            }

            @Override
            public long getContentLengthLong() {
                return reqBodyBytes.length;
            }
        });
    }

    /**
     * @param map map
     * @return boolean true 有空值 false 没有空值
     * @description 校验map中是否有空值
     * @date 2020-01-16 16:49
     */
    public static boolean verifyMapHasNullValue(Map map) {
        //map为空直接返回
        if (map == null || map.size() == 0) {
            return true;
        }
        //map中没
        Collection valueList = map.values();
        for (Object value : valueList) {
            if (value == null) {
                return true;
            }
        }
        return false;
    }

}
